@@ -27,7 +27,8 @@
 #include "librpc/gen_ndr/libnet_join.h"
 #include "libnet/libnet_join.h"
 #include "../libcli/auth/libcli_auth.h"
-#include "../librpc/gen_ndr/srv_wkssvc.h"
+#include "librpc/gen_ndr/ndr_wkssvc.h"
+#include "librpc/gen_ndr/ndr_wkssvc_scompat.h"
 #include "../libcli/security/security.h"
 #include "session.h"
 #include "smbd/smbd.h"
@@ -79,7 +80,8 @@ static char **get_logged_on_userlist(TALLOC_CTX *mem_ctx)
 		for (i = 0; i < num_users; i++) {
 			/* getutxent can return multiple user entries for the
 			 * same user, so ignore any dups */
-			if (strcmp(u->ut_user, usr_infos[i].name) == 0) {
+			int cmp = strncmp(u->ut_user, usr_infos[i].name, sizeof(u->ut_user));
+			if (cmp == 0) {
 				break;
 			}
 		}
@@ -345,6 +347,8 @@ static struct wkssvc_NetWkstaInfo102 *create_wks_info_102(TALLOC_CTX *mem_ctx)
 WERROR _wkssvc_NetWkstaGetInfo(struct pipes_struct *p,
 			       struct wkssvc_NetWkstaGetInfo *r)
 {
+	struct dom_sid_buf buf;
+
 	switch (r->in.level) {
 	case 100:
 		/* Level 100 can be allowed from anyone including anonymous
@@ -362,8 +366,9 @@ WERROR _wkssvc_NetWkstaGetInfo(struct pipes_struct *p,
 				 "101\n"));
 			DEBUGADD(3,(" - does not have sid for Authenticated "
 				    "Users %s:\n",
-				    sid_string_dbg(
-					    &global_sid_Authenticated_Users)));
+				    dom_sid_str_buf(
+					    &global_sid_Authenticated_Users,
+					    &buf)));
 			security_token_debug(DBGC_CLASS, 3,
 					    p->session_info->security_token);
 			return WERR_ACCESS_DENIED;
@@ -381,7 +386,9 @@ WERROR _wkssvc_NetWkstaGetInfo(struct pipes_struct *p,
 				 "102\n"));
 			DEBUGADD(3,(" - does not have sid for Administrators "
 				    "group %s, sids are:\n",
-				    sid_string_dbg(&global_sid_Builtin_Administrators)));
+				    dom_sid_str_buf(
+					    &global_sid_Builtin_Administrators,
+					    &buf)));
 			security_token_debug(DBGC_CLASS, 3,
 					    p->session_info->security_token);
 			return WERR_ACCESS_DENIED;
@@ -563,10 +570,13 @@ WERROR _wkssvc_NetWkstaEnumUsers(struct pipes_struct *p,
 	/* This with any level should only be allowed from a domain administrator */
 	if (!nt_token_check_sid(&global_sid_Builtin_Administrators,
 				p->session_info->security_token)) {
+		struct dom_sid_buf buf;
 		DEBUG(1,("User not allowed for NetWkstaEnumUsers\n"));
 		DEBUGADD(3,(" - does not have sid for Administrators group "
-			    "%s\n", sid_string_dbg(
-				    &global_sid_Builtin_Administrators)));
+			    "%s\n",
+			    dom_sid_str_buf(
+				    &global_sid_Builtin_Administrators,
+				    &buf)));
 		security_token_debug(DBGC_CLASS, 3, p->session_info->security_token);
 		return WERR_ACCESS_DENIED;
 	}
@@ -1061,3 +1071,6 @@ WERROR _wkssvc_NetrEnumerateComputerNames(struct pipes_struct *p,
 	p->fault_state = DCERPC_FAULT_OP_RNG_ERROR;
 	return WERR_NOT_SUPPORTED;
 }
+
+/* include the generated boilerplate */
+#include "librpc/gen_ndr/ndr_wkssvc_scompat.c"
